{-|
    A module which implements the calculation of apparent rates in a
    pepa model.
-}
module Language.Pepa.ApparentRates
    ( summedPepaRate
    )
where

{- Standard Library Modules Imported -}
import Data.List
   ( partition )
import Data.Maybe
   ( maybe )
{- Non-Standard Library Modules Imported -}
{- Local Library Modules Imported -}
import Language.Pepa.Syntax 
   ( ParsedModel
   , ParsedComponentId
   , ParsedTrans         ( .. )
   , ParsedAction
   , ParsedRate          ( .. )
   , ParsedRateExp       ( .. )
   , isPassiveRate
   , isTimedRate
   )
import Language.Pepa.PepaUtils
   ( possibleTransitions )
{- End of Module Imports -}

{-| 
    Calculates the summed rate of an action 
-}
summedPepaRate :: ParsedAction -> ParsedComponentId 
               -> ParsedModel -> ParsedRate
summedPepaRate act component model =
   case splitRates of
      ([], [])               -> error "No rates to sum"
      ([], immediates)       -> sumImmediates immediates
      (rateExps, [])
         | all isPassiveRate rateExps -> RateTop
         | any isPassiveRate rateExps -> error "mixed passive and active"
         | otherwise                  -> sumRateExps rateExps
      (_, _)                ->
         error "Both timed and immediate actions of the same action kind"
   where
   trans      = filter ((act==) . pepaTransAction) $   
                  possibleTransitions model component
   rates      = map pepaTransRate trans

   splitRates = partition isTimedRate rates

   sumImmediates :: [ ParsedRate ] -> ParsedRate
   sumImmediates immediates =
      RateImmediate $ Just weight
      where
      -- the call to maybe just returns the weight contained in the Maybe
      -- or 1.0 if the weight is 'Nothing'
      weight = foldr1 (+) [ maybe 1.0 id m | RateImmediate m <- immediates ]

   sumRateExps :: [ ParsedRate ] -> ParsedRate
   sumRateExps rateExps =
      RateTimed expr
      where
      expr = foldr1 (RateBexp "+") [ e | RateTimed e <- rateExps ]

   
