{-| 
   This module implements the adding of probe components into the
   model. This also calls the translation routine found in
   'Language.Pepa.Probes.Translate'

   Actually you could say this is slightly bad design perhaps this should
   only do the actual addition, that is it should take in a pepa component
   to add to the model. This would of course be more general.
   If you are reading this and want this, give me a shout.
-}
module Language.Pepa.Probes.AddProbes
   ( -- * Adding the translated probe to the model
     addMasterProbe
   , addProbeDefs
   , addGlobalProbe
   , addLocalProbe

     -- * Re-exported from 'Language.Pepa.Probes.Translate'
   , ProbeTranslateFlags     ( .. )
   )
where

{- Imported Standard Libraries -}
import Data.List
    ( find )
import Data.Set
   ( toList )
{- Imported External Libraries -}
{- Imported Local Libraries -}
import Language.Pepa.Syntax
    ( QualifiedName     ( .. )
    , ActionIdentifier
    , ParsedComponentId
    , ProcessDef
    , ParsedModel       ( .. )
    , ParsedComponent   ( .. )
    , ParsedRate        ( .. )
    , isTimedRate
    , ParsedAction      ( .. )
    , ParsedTrans       ( .. )
    , defaultPepaPriority
    )
import Language.Pepa.PepaUtils
   ( addProcessDefinitions
   , allTransitionsOfModel
   )
import Language.Pepa.Transform.NameSupply
    ( NameSupply
    , getName
    , initialNameSupply
    )

-- Importing the 'ProbeR' data type from here is temporary
-- until I figure out the nice module structure for probes.
import Language.Pepa.Probes.StochasticProbes
   ( ProbeDef
   , probeAlphabet
   )
import Language.Pepa.Probes.Translate
   ( translateProbeR
   , ProbeTranslateFlags     ( .. )
--   , comActionPriority
   )
{- End of Imports -}

{-| 
   Add the master probe, this is always the same probe.
   It has just two states @stopped@ and @running@.
-}
addMasterProbe :: [ String ] -> [ String ] -> ParsedModel -> ParsedModel
addMasterProbe startNames stopNames pmod =
   addProcessDefinitions masterDefs $ 
      addGlobalProbe masterActions masterComp pmod
   where
   masterComp    = IdProcess stoppedId
   masterDefs    = masterProbeDefinitions startTrans stopTrans 
                                          stoppedId runningId
   stoppedId     = Unqualified "ProbeStopped"
   runningId     = Unqualified "ProbeRunning"
   masterActions = startActs ++ stopActs

   startIds      = map Unqualified startNames
   stopIds       = map Unqualified stopNames

   startActs     = map pepaTransAction startTrans
   stopActs      = map pepaTransAction stopTrans   

   startTrans    = map (makeProbeTransition pmod) startIds
   stopTrans     = map (makeProbeTransition pmod) stopIds


{-| The master probe definitions are very simple.
    There are two states, running and stopped, and we
    synchronise only over the special communication actions
    /start/ and /stop/.
    This function takes in the stopped and running state
    names.
-}
masterProbeDefinitions :: [ ParsedTrans ] -> [ ParsedTrans ]
                       -> ParsedComponentId -> ParsedComponentId
                       -> [ ProcessDef ]
masterProbeDefinitions [] _ _ _                                     =
   error "You must provide some start actions"
masterProbeDefinitions _ [] _ _                                     =
   error "You must provide some stop actions"
masterProbeDefinitions startTrans stopTrans stoppedName runningName =
   [ stoppedDef, runningDef ]
   where
   stoppedDef  = (stoppedName, stoppedProc)
   runningDef  = (runningName, runningProc)
   -- Astute readers will notice that they are both the same
   -- process.
   stoppedProc = foldr1 ComponentSum allPrefixes
   runningProc = foldr1 ComponentSum allPrefixes

   allPrefixes = (map mkStartPrefix startTrans) ++
                 (map mkStopPrefix stopTrans)

   mkStartPrefix :: ParsedTrans -> ParsedComponent
   mkStartPrefix t = PrefixComponent t runningIdP

   mkStopPrefix :: ParsedTrans -> ParsedComponent
   mkStopPrefix t = PrefixComponent t stoppedIdP

   stoppedIdP  = IdProcess stoppedName
   runningIdP  = IdProcess runningName


{-| Translate and add a series of probe definitions to a model -}
addProbeDefs :: [ ProbeTranslateFlags ] -> [ ProbeDef ]
             -> ParsedModel -> Either String ParsedModel
addProbeDefs trFlags = 
   translateAndAdd initialNameSupply
   where
   translateAndAdd :: NameSupply -> [ ProbeDef ] -> ParsedModel
                   -> Either String ParsedModel
   translateAndAdd _ns [] model                        =
      Right model
   translateAndAdd ns ((mPattach, probe) : rest) model = 
      case mPattach of
         Nothing        ->
            translateAndAdd newNs rest probeModel
            where
            probeModel  = addProcessDefinitions pDefs probeModel1
            probeModel1 = addGlobalProbe actions probeComp model        
         Just attachId  ->
            case addLocalProbe  actions probeComp attachId model of
               Nothing   -> Left "Local probe could not be attached"
               Just pMod -> translateAndAdd newNs rest $
                              addProcessDefinitions pDefs pMod
      where
      pDefs       = translateProbeR trFlags startName transits probe
      -- translateProbe probe newNs startId stopId rPairs transits []
      
      probeComp   = IdProcess startId
      startId     = Unqualified $ startName
      newNs       = fst startNSRes
      startNSRes  = getName ns "Probe_" 
      startName   = (snd startNSRes) ++ "_start_state_yy"
      actionIds   = toList $ probeAlphabet probe
      transits    = map (makeProbeTransition model) actionIds
      actions     = map pepaTransAction transits




-- When creating the probe tracker it must know whether to add the
-- passive actions at rate 'infty' or at rate 'immediate' so this
-- function takes in an action and returns a parsed trans that can
-- be used in the probe tracker
makeProbeTransition :: ParsedModel -> ActionIdentifier -> ParsedTrans
makeProbeTransition model a =
   case find isRightAction $ allTransitionsOfModel model of
      Nothing                            ->
         ParsedTrans { pepaTransAction   = Action a
                     , pepaTransRate     = RateTop
                     , pepaTransPriority = defaultPepaPriority
                     }
      Just t
         | isTimedRate $ pepaTransRate t ->
            t { pepaTransRate = RateTop }
         | otherwise                     -> t
   where
   isRightAction :: ParsedTrans -> Bool
   isRightAction (ParsedTrans { pepaTransAction = Action a2 })    = a == a2
   isRightAction (ParsedTrans { pepaTransAction = ComAction a2 }) = a == a2
   isRightAction _                                                = False

{-| Adding a global probe is just the addition of the cooperation
    with the main system equation.
-}
addGlobalProbe :: [ ParsedAction ] -> ParsedComponent 
              -> ParsedModel -> ParsedModel
addGlobalProbe actions probeComp (ParsedModel rDefs pDefs mainCoop) =
   ParsedModel rDefs pDefs cooperation
   where
   cooperation = Cooperation mainCoop actions probeComp
   {- actions     = map Action actionIds -}

{-| The addition of a local probe to a model.
    Here the probe is attached in cooperation with the given component
    name.
    This must essentially do breadth first search on the model
    (it doesn't necessarily have to be breadth first but if it is done
    this way then the modeller see his/her probe attached at the
    highest possible level.) Note that it is possible that it makes
    a difference, but then the person has not defined their model/probe
    very well.
    [@todo@] We do not actually do breadth first search, we just attempt
    to add it to the system equation and if that's not possible then we
    just attempt to add it to one of the process definitions, making
    sure that it is a parallel definition. This can actually result in
    an error. If we add it to a process definition which is used in a
    sequential position. Consider
    @
    P = (a, r).Q ;
    Q = (b, r).S ;
    S = Q
    S <a> S
    @
    If we attempt to add the probe to @Q@ we will get the model
    @
    P = (a, r).S ;
    Q = (b, r).P ;
    S = Q <a> Probe
    S <a> S
    @
    but @S@ is used in the sequential position in the definition
    of @P@. What would be nice is if the typing phase allowed the
    splitting of the definitions into parallel and sequential definitions.
    For now, provided that typing is done after the probe is added this
    will suffice.
-}
addLocalProbe :: [ ParsedAction ] -> ParsedComponent 
              -> ParsedComponentId -> ParsedModel -> Maybe ParsedModel
addLocalProbe actions probeComp attachId (ParsedModel rDefs pDefs mainCoop) =
   case addToCoop mainCoop of
      Just newCoop -> return $ ParsedModel rDefs pDefs newCoop
      -- Notice we reverse the defs and then re-reverse them, this
      -- just means it is more likely that a definition near the
      -- system equation will be used to add the probe, lessening
      -- the chance that the above bug will occur.
      Nothing      -> do newDefs <- addToProcessDefs $ reverse pDefs
                         return $ ParsedModel rDefs (reverse newDefs) mainCoop
   where
   -- We return the new component if it was possible to attach the
   -- the probe.
   addToCoop :: ParsedComponent -> Maybe ParsedComponent
   addToCoop (IdProcess ident)
      | attachId == ident = Just cooperation
      | otherwise         = Nothing
   addToCoop (Cooperation left acts right)  =
      case addToCoop left of
        Nothing      -> do newRight <- addToCoop right
                           return $ Cooperation left acts newRight
        Just newLeft -> return $ Cooperation newLeft acts right
   addToCoop (ProcessArray ident size mActs)
      | attachId == ident = Just comp
      | otherwise         = Nothing
      where
      comp      = Cooperation cooperation arrayActs newArray
      arrayActs = maybe [] id mActs
      newArray  = if size <= 2
                  then IdProcess ident
                  else ProcessArray ident (size - 1) mActs
   addToCoop (Hiding comp hiddenActs)      =
      do newComp <- addToCoop comp
         return $ Hiding newComp hiddenActs
   -- If it's a sequential component then we cannot add it.
   addToCoop _                             =
      Nothing

   -- This is not exactly what we want as we would rather it add
   -- to something that was specifically in the main coop.
   addToProcessDefs :: [ ProcessDef ] -> Maybe [ ProcessDef ]
   addToProcessDefs []         = Nothing
   addToProcessDefs ((name, comp) : rest) =
      case addToCoop comp of
         Nothing -> do newRest <- addToProcessDefs rest
                       return $ (name, comp) : newRest
         Just c  -> Just $ (name, c) : rest

   -- The cooperation will always be the same
   cooperation = Cooperation (IdProcess attachId) actions probeComp
   {- actions     = map Action actionIds -}
