{-|
   A printing module for Prism files.
-}
module Language.Prism.Print
   ( printPrismModelFile
   )
where


{- Imported Standard Libraries -}
{- Imported Local Libraries -}
import Language.Pepa.Print
   ( HumanPrint          ( .. ) )
import Language.Prism.Syntax
   ( PrismModelFile      ( .. )
   , PrismModel          ( .. )

   , PrismModelKind      ( .. )
   , PrismModule         ( .. )
   , PrismVariable       ( .. )
   , PrismTransition     ( .. )
   , PrismCondition  --  ( .. )
   , PrismAction     --  ( .. )
   , PrismRate
   )
import Language.Hydra.Syntax
   ( DNAMcondition       ( .. )
   , DNAMcassignment     ( .. )
   , DNAMspeed           ( .. )
   )
import Language.Hydra.Print
   ( printCexp
   )
import Language.Pepa.Utils
   ( mkSepList )
{- End of Imports -}


{-| The main exported printing function of this module.
    Prints a prism model file.
-}
printPrismModelFile :: PrismModelFile -> String
printPrismModelFile (PrismModelFile model) =
   printPrismModel model

printPrismModel :: PrismModel -> String
printPrismModel (PrismModel kind modules) =
   unlines ( printPrismModelKind kind :
             (map printPrismModule modules) )
          

printPrismModelKind :: PrismModelKind -> String
printPrismModelKind PrismMdm  = "mdm"
printPrismModelKind PrismDtmc = "dtmc"
printPrismModelKind PrismCtmc = "ctmc"

printPrismModule :: PrismModule -> String
printPrismModule (PrismModule name variables transitions) =
   unlines $ concat [ [ headLine ]
                    , map printPrismVariable variables
                    , map printPrismTransition transitions
                    , [ "endmodule" ]
                    ]
   where
   headLine = "module " ++ (hprint name)

printPrismVariable :: PrismVariable -> String
printPrismVariable variable =
   unwords [ hprint $ prismVarname variable
           , "["
           , show $ prismVarLow variable
           , ".."
           , show $ prismVarHigh variable
           , "]"
           , "init"
           , show $ prismVarInit variable
           , ";"
           ]
   
printPrismTransition :: PrismTransition -> String
printPrismTransition transition =
   unwords [ "[]"
           , conditions
           , "->"
           , printPrismSpeed $ prismTransRate transition
           , ":"
           , actions
           , ";"
           ]
   where
   conditions    = unwords condStrings
   condStrings   = map printPrismCondition $ prismTransGuard transition
   actions       = mkSepList " & " actionStrings
   actionStrings = map printPrismAction $ prismTransActions transition


printPrismSpeed :: PrismRate -> String
printPrismSpeed (DNAMrate cexp) = printCexp cexp
printPrismSpeed (DNAMweight _)  =
   error "Immediate actions unimplemented in Prism compilation"

printPrismCondition :: PrismCondition -> String
printPrismCondition (DNAMcond cexp)         = printCexp cexp
printPrismCondition (DNAMprocpresent ident) =
   (hprint ident) ++ " > 0"


printPrismAction :: PrismAction -> String
printPrismAction (DNAMidentIncr ident) =
   updateVariable (hprint ident) "+ 1"
printPrismAction (DNAMidentDecr ident) =
   updateVariable (hprint ident) "- 1"
printPrismAction (DNAMcassignment _ _) =
   error "Unimplemented prism update"

-- Creates a prism variable updated with the second string.
updateVariable :: String -> String -> String
updateVariable ident oper =
   unwords [ ident, "=", ident, oper ]