#!/usr/bin/env runhaskell
import Distribution.Simple
   ( defaultUserHooks
   , UserHooks
   , defaultMainWithHooks   
   )
import Distribution.PackageDescription
   ( readPackageDescription 
   , PackageDescription
   , libModules
   )
import System.Cmd  ( system )
import System.Exit ( exitWith )
import System      ( getArgs )


myHooks :: UserHooks
myHooks = defaultUserHooks


main :: IO ()
main = do args <- getArgs
          processArgs args


processArgs :: [ String ] -> IO ()
processArgs [ "regTests" ]         =
    do _exitCode  <- system "runhaskell tests/RunTests.hs"
       return ()
processArgs ( "quickCheck" : rest) =
   do _exitCode   <- system qcCommand
      return ()
   where
   qcCommand :: String
   qcCommand =  unwords ("runhaskell tests/quickcheck/Properties.hs" : rest)

-- Add syntax-coloring and anchors to sources. 
-- Use CSS and copy in 'hscolour-css'.
processArgs ( "hs-colour" : rest)  =
   do _exitCode <- system mkdirCommand
      writeFile "dist/doc/html/hscolour.css" hscolourContents
      pkgDesc   <- readPackageDescription "ipclib.cabal"
      let modules = libModules pkgDesc
      mapM_ hsColourModule modules
   where
   mkdirCommand   = unwords [ "mkdir", "-p", "dist/doc/html" ]

   hsColourModule :: String -> IO ()
   hsColourModule moduleName =
      do _exitCode <- system command
         return ()
      where
      command = unwords [ hsColour
                        , "-css -anchor"
                        , moduleFileName
                        , "> dist/doc/html/" ++ moduleName ++ ".hs.html"
                        ]
      moduleFileName :: FilePath
      moduleFileName = 
         subst ++ ".hs"
         where
         subst = map (\x -> if x == '.' then '/' else x) moduleName
processArgs _           = defaultMainWithHooks myHooks


-- Should also be able to set this in the command-line arguments
hsColour :: String
hsColour = "HsColour"

-- We should allow the user to specify an hscolour css file 
hscolourContents :: String
hscolourContents =
   unlines [ ""
           , ".keyglyph, .layout {color: red;}"
           , ".keyword {color: blue;}"
           , ".comment, .comment a {color: green;}"
           , ".str, .chr {color: teal;}"
           , ".keyword,.conid, .varid, .conop, .varop, .num, .cpp, .sel {}"
           , ""
           ]

{-
# Generate links to colorized source code.  See source-args comment below.
source-args ?= \
	  --source-module=$(top-src-dot)%{MODULE}.hs.html \
	  --source-entity=%{FILE///.}.html\#%{NAME}
-}
