{-|
   This is a very simple model which augments the main ipc command-line
   options with those specific to smc
-}
module Smc.Cli
   ( smcVersion
   , SmcOptions   ( .. )
   , smcOptions
   , srmcManual
   )
where

{- Standard library modules imported -}
import System.Console.GetOpt
   ( OptDescr   ( .. )
   , ArgDescr   ( .. )
   )
{- External Library modules imported -}
{- Local modules imported -}
import Ipc.Cli
   ( CliOpt         ( .. )
   , baseCliOptions
   , Manual
   , ManualSection
   , ManualDesc
   , ManualEntry
   , ipcManual
   , optDescsOfManual
   )
{- End of module imports -}

smcVersion :: String
smcVersion = "This is smc version 0.01"

{-| The kind of options available in the smc compiler /in addition/ to those
   available to generic ipc commands.
-}
data SmcOptions =
   PepaCondorDir FilePath
 | IpcArgs String
 | IpcLocation FilePath
   deriving Eq

smcOptions :: [ OptDescr ( CliOpt SmcOptions ) ]
smcOptions = 
  optDescsOfManual srmcManual

srmcManual :: Manual SmcOptions
srmcManual = ipcManual ++ srmcOnlyManual


srmcOnlyManual :: Manual SmcOptions
srmcOnlyManual =
   [ (header, entries) ]
   where
   header = unlines [ "\\section{Smc specific options}"
                    , "The options in this section apply only to the"
                    , "\\srmc compiler and are not related to equivalent"
                    , "options in the \\ipc\\ compiler."
                    ]
   entries = 
    [ ( unlines [ "The \\srmc\\ compiler will generate many files"
                , "By default these will be placed in the same directory"
                , "as that which contains the source \\texttt{.srmc} file."
                , "The option \\ipcflag{pepa-condor-dir}"
                , "allows the user to specify a separate directory"
                , "into which to place all of the generated files"
                ]
      , Option ""     [ "pepa-condor-dir" ] 
        (ReqArg (CliNonStandard . PepaCondorDir) "Dir")
        "The directory in which to store all the produced pepa models"
      )

    , ( unlines [ "By default the \\srmc compiler will assume that"
                , "the \\ipc\\ compiler is installed in a globally visible"
                , "location (on Unix systems in the user's PATH)."
                , "If \\ipc\\ is installed elsewhere the user may"
                , "point directly to the location with the"
                , "\\ipcflag{ipc} option."
                ]
      , Option ""     [ "ipc" ]
        (ReqArg (CliNonStandard . IpcLocation) "PATH")
        "A possibly non-standard location of the ipc executable to run"
      )

    , ( unlines [ "The \\srmc\\ compiler generates batch files to"
                , "invoke the \\ipc\\ compiler over the generated"
                , "\\pepa\\ files. Should the user wish to pass"
                , "additional flags to the \\ipc\\ compiler the"
                , "\\ipcflag{ipc-args} flag allows for this."
                ]
      , Option ""     [ "ipc-args" ]
        (ReqArg (CliNonStandard . IpcArgs) "argument string")
        ( "A string containing arguments which will be passed on to ipc"
          ++ " when for example outputting a batch script to solve all the"
          ++ " generated pepa models" )
      )
    ]
