{-|
    This is a simple example use of the Pepa library.
-}

module Main
   ( main )
where

{- Standard library modules imported -}
import Data.Maybe
   ( isJust )
import System.Environment 
   ( getArgs )
import System.Exit
   ( exitWith
   , ExitCode      ( .. )
   , exitFailure
   )
import Text.ParserCombinators.Parsec
   ( ParseError )

{- Local modules imported -}
import Language.Pepa.Syntax
   ( ParsedModel ) 
import Language.Pepa.Parser
   ( parsePepaFile )
import Language.Pepa.Analysis.Analysis
   ( analyseModel
   , messagesOfReport
   )
import Language.Pepa.Print
   ( hprintMessages )
{- End of module imports -}


main :: IO ()
main = do args         <- getArgs
          processArgs args
          
processArgs :: [ String ] -> IO () 
processArgs []       = putStrLn "No input files to analyse"
processArgs files    = do errors <- mapM analyseFile files
                          if any isJust errors
                             then exitFailure
                             else exitWith ExitSuccess

analyseFile :: FilePath -> IO (Maybe String)
analyseFile file = 
    do contents <- readFile file
       let pResult  = parsePepaFile file contents
           mMessage = interpretParseResult pResult
           message            = 
            case mMessage of
              Nothing  -> "model passed with no warnings or errors"
              Just m   -> m
       putStrLn $ "Analysing file " ++ file
       putStrLn $ message
       putStrLn "---------------------"
       return mMessage


interpretParseResult :: Either ParseError ParsedModel -> Maybe String
interpretParseResult (Left err)     = Just $ show err
interpretParseResult (Right model)  = 
    case messagesOfReport $ analyseModel model of
      [] -> Nothing
      l  -> Just $ hprintMessages l
